@@ -27,7 +27,8 @@ import six
 
 from oslo.vmware.common import loopingcall
 from oslo.vmware import exceptions
-from oslo.vmware.openstack.common.gettextutils import _
+from oslo.vmware.openstack.common import excutils
+from oslo.vmware.openstack.common.gettextutils import _, _LE, _LI, _LW
 from oslo.vmware import pbm
 from oslo.vmware import vim
 from oslo.vmware import vim_util
@@ -36,6 +37,12 @@ from oslo.vmware import vim_util
 LOG = logging.getLogger(__name__)
 
 
+def _trunc_id(session_id):
+    """Returns truncated session id which is suitable for logging."""
+    if session_id is not None:
+        return session_id[-5:]
+
+
 # TODO(vbala) Move this class to excutils.py.
 class RetryDecorator(object):
     """Decorator for retrying a function upon suggested exceptions.
@@ -77,45 +84,49 @@ class RetryDecorator(object):
 
         def _func(*args, **kwargs):
             func_name = f.__name__
+            result = None
             try:
-                LOG.debug(_("Invoking %(func_name)s; retry count is "
-                            "%(retry_count)d."),
+                LOG.debug("Invoking %(func_name)s; retry count is "
+                          "%(retry_count)d.",
                           {'func_name': func_name,
                            'retry_count': self._retry_count})
                 result = f(*args, **kwargs)
-                LOG.debug(_("Function %(func_name)s returned successfully "
-                            "after %(retry_count)d retries."),
+                LOG.debug("Function %(func_name)s returned successfully "
+                          "after %(retry_count)d retries.",
                           {'func_name': func_name,
                            'retry_count': self._retry_count})
-            except self._exceptions as excep:
-                LOG.warn(_("Exception which is in the suggested list of "
-                           "exceptions occurred while invoking function:"
-                           " %s."),
-                         func_name,
-                         exc_info=True)
-                if (self._max_retry_count != -1 and
-                        self._retry_count >= self._max_retry_count):
-                    LOG.error(_("Cannot retry upon suggested exception since "
-                                "retry count (%(retry_count)d) reached "
-                                "max retry count (%(max_retry_count)d)."),
-                              {'retry_count': self._retry_count,
-                               'max_retry_count': self._max_retry_count})
-                    raise excep
-                else:
-                    self._retry_count += 1
-                    self._sleep_time += self._inc_sleep_time
-                    return self._sleep_time
-            except Exception as excep:
-                LOG.exception(_("Exception which is not in the suggested list "
-                                "of exceptions occurred while invoking %s."),
-                              func_name)
-                raise excep
+            except self._exceptions:
+                with excutils.save_and_reraise_exception() as ctxt:
+                    LOG.warn(_LW("Exception which is in the suggested list of "
+                                 "exceptions occurred while invoking function:"
+                                 " %s."),
+                             func_name,
+                             exc_info=True)
+                    if (self._max_retry_count != -1 and
+                            self._retry_count >= self._max_retry_count):
+                        LOG.error(_LE("Cannot retry upon suggested exception "
+                                      "since retry count (%(retry_count)d) "
+                                      "reached max retry count "
+                                      "(%(max_retry_count)d)."),
+                                  {'retry_count': self._retry_count,
+                                   'max_retry_count': self._max_retry_count})
+                    else:
+                        ctxt.reraise = False
+                        self._retry_count += 1
+                        self._sleep_time += self._inc_sleep_time
+                        return self._sleep_time
+            except Exception:
+                with excutils.save_and_reraise_exception():
+                    LOG.exception(_LE("Exception which is not in the "
+                                      "suggested list of exceptions occurred "
+                                      "while invoking %s."),
+                                  func_name)
             raise loopingcall.LoopingCallDone(result)
 
         def func(*args, **kwargs):
             loop = loopingcall.DynamicLoopingCall(_func, *args, **kwargs)
             evt = loop.start(periodic_interval_max=self._max_sleep_time)
-            LOG.debug(_("Waiting for function %s to return."), f.__name__)
+            LOG.debug("Waiting for function %s to return.", f.__name__)
             return evt.wait()
 
         return func
@@ -125,18 +136,21 @@ class VMwareAPISession(object):
     """Setup a session with the server and handles all calls made to it.
 
     Example:
-        api_session = VMwareAPISession('10.1.2.3', 'administrator', 'password',
-                                       10, 0.1, create_session=False)
+        api_session = VMwareAPISession('10.1.2.3', 'administrator',
+                                       'password', 10, 0.1,
+                                       create_session=False, port=443)
         result = api_session.invoke_api(vim_util, 'get_objects',
                                         api_session.vim, 'HostSystem', 100)
     """
 
     def __init__(self, host, server_username, server_password,
                  api_retry_count, task_poll_interval, scheme='https',
-                 create_session=True, wsdl_loc=None, pbm_wsdl_loc=None):
+                 create_session=True, wsdl_loc=None, pbm_wsdl_loc=None,
+                 port=443):
         """Initializes the API session with given parameters.
 
-        :param host: ESX/VC server IP address[:port] or host name[:port]
+        :param host: ESX/VC server IP address or host name
+        :param port: port for connection
         :param server_username: username of ESX/VC server admin user
         :param server_password: password for param server_username
         :param api_retry_count: number of times an API must be retried upon
@@ -152,6 +166,7 @@ class VMwareAPISession(object):
                  VimSessionOverLoadException
         """
         self._host = host
+        self._port = port
         self._server_username = server_username
         self._server_password = server_password
         self._api_retry_count = api_retry_count
@@ -171,20 +186,22 @@ class VMwareAPISession(object):
         if not self._vim:
             self._vim = vim.Vim(protocol=self._scheme,
                                 host=self._host,
-                                wsdl_loc=self._vim_wsdl_loc)
+                                port=self._port,
+                                wsdl_url=self._vim_wsdl_loc)
         return self._vim
 
     @property
     def pbm(self):
         if not self._pbm and self._pbm_wsdl_loc:
-            self._pbm = pbm.PBMClient(self._pbm_wsdl_loc,
-                                      protocol=self._scheme,
-                                      host=self._host)
+            self._pbm = pbm.Pbm(protocol=self._scheme,
+                                host=self._host,
+                                port=self._port,
+                                wsdl_url=self._pbm_wsdl_loc)
             if self._session_id:
                 # To handle the case where pbm property is accessed after
                 # session creation. If pbm property is accessed before session
                 # creation, we set the cookie in _create_session.
-                self._pbm.set_cookie(self._get_session_cookie())
+                self._pbm.set_soap_cookie(self._vim.get_http_cookie())
         return self._pbm
 
     @RetryDecorator(exceptions=(exceptions.VimConnectionException,))
@@ -192,7 +209,7 @@ class VMwareAPISession(object):
         """Establish session with the server."""
         session_manager = self.vim.service_content.sessionManager
         # Login and create new session with the server for making API calls.
-        LOG.debug(_("Logging in with username = %s."), self._server_username)
+        LOG.debug("Logging in with username = %s.", self._server_username)
         session = self.vim.Login(session_manager,
                                  userName=self._server_username,
                                  password=self._server_password)
@@ -203,15 +220,16 @@ class VMwareAPISession(object):
         # object. We can't use the username used for login since the Login
         # method ignores the case.
         self._session_username = session.userName
-        LOG.info(_("Successfully established new session; session ID is %s."),
-                 self._session_id)
+        LOG.info(_LI("Successfully established new session; session ID is "
+                     "%s."),
+                 _trunc_id(self._session_id))
 
         # Terminate the previous session (if exists) for preserving sessions
         # as there is a limit on the number of sessions we can have.
         if prev_session_id:
             try:
-                LOG.info(_("Terminating the previous session with ID = %s"),
-                         prev_session_id)
+                LOG.info(_LI("Terminating the previous session with ID = %s"),
+                         _trunc_id(prev_session_id))
                 self.vim.TerminateSession(session_manager,
                                           sessionId=[prev_session_id])
             except Exception:
@@ -220,30 +238,31 @@ class VMwareAPISession(object):
                 # have been cleared already. We could have made a call to
                 # SessionIsActive, but that is an overhead because we
                 # anyway would have to call TerminateSession.
-                LOG.warn(_("Error occurred while terminating the previous "
-                           "session with ID = %s."),
-                         prev_session_id,
+                LOG.warn(_LW("Error occurred while terminating the previous "
+                             "session with ID = %s."),
+                         _trunc_id(prev_session_id),
                          exc_info=True)
 
         # Set PBM client cookie.
         if self._pbm is not None:
-            self._pbm.set_cookie(self._get_session_cookie())
+            self._pbm.set_soap_cookie(self._vim.get_http_cookie())
 
-    def __del__(self):
+    def logout(self):
         """Log out and terminate the current session."""
         if self._session_id:
-            LOG.info(_("Logging out and terminating the current session with "
-                       "ID = %s."),
-                     self._session_id)
+            LOG.info(_LI("Logging out and terminating the current session "
+                         "with ID = %s."),
+                     _trunc_id(self._session_id))
             try:
                 self.vim.Logout(self.vim.service_content.sessionManager)
+                self._session_id = None
             except Exception:
-                LOG.exception(_("Error occurred while logging out and "
-                                "terminating the current session with "
-                                "ID = %s."),
-                              self._session_id)
+                LOG.exception(_LE("Error occurred while logging out and "
+                                  "terminating the current session with "
+                                  "ID = %s."),
+                              _trunc_id(self._session_id))
         else:
-            LOG.debug(_("No session exists to log out."))
+            LOG.debug("No session exists to log out.")
 
     def invoke_api(self, module, method, *args, **kwargs):
         """Wrapper method for invoking APIs.
@@ -265,7 +284,7 @@ class VMwareAPISession(object):
                         exceptions=(exceptions.VimSessionOverLoadException,
                                     exceptions.VimConnectionException))
         def _invoke_api(module, method, *args, **kwargs):
-            LOG.debug(_("Invoking method %(module)s.%(method)s."),
+            LOG.debug("Invoking method %(module)s.%(method)s.",
                       {'module': module,
                        'method': method})
             try:
@@ -283,8 +302,8 @@ class VMwareAPISession(object):
                     # case of an inactive session. Therefore, we need a way to
                     # differentiate between these two cases.
                     if self._is_current_session_active():
-                        LOG.debug(_("Returning empty response for "
-                                    "%(module)s.%(method)s invocation."),
+                        LOG.debug("Returning empty response for "
+                                  "%(module)s.%(method)s invocation.",
                                   {'module': module,
                                    'method': method})
                         return []
@@ -294,7 +313,7 @@ class VMwareAPISession(object):
                             _("Current session: %(session)s is inactive; "
                               "re-creating the session while invoking "
                               "method %(module)s.%(method)s.") %
-                            {'session': self._session_id,
+                            {'session': _trunc_id(self._session_id),
                              'module': module,
                              'method': method})
                         LOG.warn(excep_msg, exc_info=True)
@@ -306,19 +325,20 @@ class VMwareAPISession(object):
                     # InvalidArgument
                     # Raise specific exceptions here if possible
                     if excep.fault_list:
-                        LOG.debug(_("Fault list: %s"), excep.fault_list)
+                        LOG.debug("Fault list: %s", excep.fault_list)
                         raise exceptions.get_fault_class(excep.fault_list[0])
                     raise
 
             except exceptions.VimConnectionException:
-                # Re-create the session during connection exception.
-                LOG.warn(_("Re-creating session due to connection problems "
-                           "while invoking method %(module)s.%(method)s."),
-                         {'module': module,
-                          'method': method},
-                         exc_info=True)
-                self._create_session()
-                raise
+                with excutils.save_and_reraise_exception():
+                    # Re-create the session during connection exception.
+                    LOG.warn(_LW("Re-creating session due to connection "
+                                 "problems while invoking method "
+                                 "%(module)s.%(method)s."),
+                             {'module': module,
+                              'method': method},
+                             exc_info=True)
+                    self._create_session()
 
         return _invoke_api(module, method, *args, **kwargs)
 
@@ -327,8 +347,8 @@ class VMwareAPISession(object):
 
         :returns: True if the session is active; False otherwise
         """
-        LOG.debug(_("Checking if the current session: %s is active."),
-                  self._session_id)
+        LOG.debug("Checking if the current session: %s is active.",
+                  _trunc_id(self._session_id))
 
         is_active = False
         try:
@@ -337,9 +357,9 @@ class VMwareAPISession(object):
                 sessionID=self._session_id,
                 userName=self._session_username)
         except exceptions.VimException:
-            LOG.warn(_("Error occurred while checking whether the "
-                       "current session: %s is active."),
-                     self._session_id,
+            LOG.warn(_LW("Error occurred while checking whether the "
+                         "current session: %s is active."),
+                     _trunc_id(self._session_id),
                      exc_info=True)
 
         return is_active
@@ -358,7 +378,7 @@ class VMwareAPISession(object):
         """
         loop = loopingcall.FixedIntervalLoopingCall(self._poll_task, task)
         evt = loop.start(self._task_poll_interval)
-        LOG.debug(_("Waiting for the task: %s to complete."), task)
+        LOG.debug("Waiting for the task: %s to complete.", task)
         return evt.wait()
 
     def _poll_task(self, task):
@@ -370,25 +390,26 @@ class VMwareAPISession(object):
 
         :param task: managed object reference of the task
         """
-        LOG.debug(_("Invoking VIM API to read info of task: %s."), task)
+        LOG.debug("Invoking VIM API to read info of task: %s.", task)
         try:
             task_info = self.invoke_api(vim_util,
                                         'get_object_property',
                                         self.vim,
                                         task,
                                         'info')
-        except exceptions.VimException as excep:
-            LOG.exception(_("Error occurred while reading info of task: %s."),
-                          task)
-            raise excep
+        except exceptions.VimException:
+            with excutils.save_and_reraise_exception():
+                LOG.exception(_LE("Error occurred while reading info of "
+                                  "task: %s."),
+                              task)
         else:
             if task_info.state in ['queued', 'running']:
                 if hasattr(task_info, 'progress'):
-                    LOG.debug(_("Task: %(task)s progress is %(progress)s%%."),
+                    LOG.debug("Task: %(task)s progress is %(progress)s%%.",
                               {'task': task,
                                'progress': task_info.progress})
             elif task_info.state == 'success':
-                LOG.debug(_("Task: %s status is success."), task)
+                LOG.debug("Task: %s status is success.", task)
                 raise loopingcall.LoopingCallDone(task_info)
             else:
                 error_msg = six.text_type(task_info.error.localizedMessage)
@@ -413,7 +434,7 @@ class VMwareAPISession(object):
         """
         loop = loopingcall.FixedIntervalLoopingCall(self._poll_lease, lease)
         evt = loop.start(self._task_poll_interval)
-        LOG.debug(_("Waiting for the lease: %s to be ready."), lease)
+        LOG.debug("Waiting for the lease: %s to be ready.", lease)
         evt.wait()
 
     def _poll_lease(self, lease):
@@ -424,26 +445,26 @@ class VMwareAPISession(object):
 
         :param lease: lease whose state is to be polled
         """
-        LOG.debug(_("Invoking VIM API to read state of lease: %s."), lease)
+        LOG.debug("Invoking VIM API to read state of lease: %s.", lease)
         try:
             state = self.invoke_api(vim_util,
                                     'get_object_property',
                                     self.vim,
                                     lease,
                                     'state')
-        except exceptions.VimException as excep:
-            LOG.exception(_("Error occurred while checking state of lease: "
-                            "%s."),
-                          lease)
-            raise excep
+        except exceptions.VimException:
+            with excutils.save_and_reraise_exception():
+                LOG.exception(_LE("Error occurred while checking "
+                                  "state of lease: %s."),
+                              lease)
         else:
             if state == 'ready':
-                LOG.debug(_("Lease: %s is ready."), lease)
+                LOG.debug("Lease: %s is ready.", lease)
                 raise loopingcall.LoopingCallDone()
             elif state == 'initializing':
-                LOG.debug(_("Lease: %s is initializing."), lease)
+                LOG.debug("Lease: %s is initializing.", lease)
             elif state == 'error':
-                LOG.debug(_("Invoking VIM API to read lease: %s error."),
+                LOG.debug("Invoking VIM API to read lease: %s error.",
                           lease)
                 error_msg = self._get_error_message(lease)
                 excep_msg = _("Lease: %(lease)s is in error state. Details: "
@@ -454,7 +475,8 @@ class VMwareAPISession(object):
             else:
                 # unknown state
                 excep_msg = _("Unknown state: %(state)s for lease: "
-                              "%(lease)s.") % {'state': state, 'lease': lease}
+                              "%(lease)s.") % {'state': state,
+                                               'lease': lease}
                 LOG.error(excep_msg)
                 raise exceptions.VimException(excep_msg)
 
@@ -467,19 +489,8 @@ class VMwareAPISession(object):
                                    lease,
                                    'error')
         except exceptions.VimException:
-            LOG.warn(_("Error occurred while reading error message for lease: "
-                       "%s."),
+            LOG.warn(_LW("Error occurred while reading error message for "
+                         "lease: %s."),
                      lease,
                      exc_info=True)
             return "Unknown"
-
-    def _get_session_cookie(self):
-        """Get the cookie corresponding to the current session.
-
-        :returns: cookie corresponding to the current session
-        """
-        cookies = self.vim.client.options.transport.cookiejar
-        for c in cookies:
-            if c.name.lower() == 'vmware_soap_session':
-                return c.value
-        return None
